
-- Source: start.sql
    
create extension dblink;


-- Source: tables/common/Channel.sql
    
CREATE TABLE Channel
(
    mgm_id                    NUMERIC NOT NULL,
    channel_id                NUMERIC NOT NULL,
    name                      VARCHAR(256),
    label                     VARCHAR(128),
    type                      VARCHAR(50),
    arch                      VARCHAR(64),
    summary                   VARCHAR(500),
    description               VARCHAR(4000),
    parent_channel_label      VARCHAR(128),
    organization              VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Channel
  ADD CONSTRAINT Channel_pk PRIMARY KEY (mgm_id, channel_id);

-- Source: tables/common/Errata.sql
    
CREATE TABLE Errata
(
    mgm_id                        NUMERIC NOT NULL,
    errata_id                     NUMERIC NOT NULL,
    advisory_name                 VARCHAR(100),
    advisory_type                 VARCHAR(32),
    advisory_status               VARCHAR(32),
    issue_date                    TIMESTAMPTZ,
    update_date                   TIMESTAMPTZ,
    severity                      VARCHAR(64),
    reboot_required               BOOLEAN NOT NULL DEFAULT FALSE,
    affects_package_manager       BOOLEAN NOT NULL DEFAULT FALSE,
    cve                           VARCHAR(4000),
    synopsis                      VARCHAR(4000),
    organization                  VARCHAR(128),
    synced_date                   TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Errata
  ADD CONSTRAINT Errata_pk PRIMARY KEY (mgm_id, errata_id);

-- Source: tables/common/Package.sql
    
CREATE TABLE Package
(
    mgm_id              NUMERIC NOT NULL,
    package_id          NUMERIC NOT NULL,
    name                VARCHAR(256),
    epoch               VARCHAR(16),
    version             VARCHAR(512),
    release             VARCHAR(512),
    arch                VARCHAR(64),
    type                VARCHAR(10),
    package_size        NUMERIC,
    payload_size        NUMERIC,
    installed_size      NUMERIC,
    vendor              VARCHAR(64),
    channel_label       VARCHAR(128),
    organization        VARCHAR(128),
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE Package
  ADD CONSTRAINT Package_pk PRIMARY KEY (mgm_id, package_id);

-- Source: tables/common/System.sql
    
CREATE TABLE System
(
    mgm_id                        NUMERIC NOT NULL,
    system_id                     NUMERIC NOT NULL,
    profile_name                  VARCHAR(128),
    hostname                      VARCHAR(128),
    minion_id                     VARCHAR(256),
    minion_os_family              VARCHAR(32),
    minion_kernel_live_version    VARCHAR(255),
    machine_id                    VARCHAR(256),
    registered_by                 VARCHAR(64),
    registration_time             TIMESTAMPTZ,
    last_checkin_time             TIMESTAMPTZ,
    kernel_version                VARCHAR(64),
    architecture                  VARCHAR(64),
    organization                  VARCHAR(128),
    hardware                      TEXT,
    synced_date                   TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE System
  ADD CONSTRAINT System_pk PRIMARY KEY (mgm_id, system_id);

CREATE INDEX System_profile_name_idx
  ON System (profile_name);

-- Source: tables/common/SystemAction.sql
    
CREATE TABLE SystemAction
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    action_id           NUMERIC NOT NULL,
    hostname            VARCHAR(128),
    pickup_time         TIMESTAMPTZ,
    completion_time     TIMESTAMPTZ,
    status              VARCHAR(16),
    event               VARCHAR(100),
    event_data          TEXT,
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemAction
  ADD CONSTRAINT SystemAction_pk PRIMARY KEY (mgm_id, system_id, action_id);

-- Source: tables/common/SystemChannel.sql
    
CREATE TABLE SystemChannel
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    channel_id                  NUMERIC NOT NULL,
    name                        VARCHAR(256),
    description                 VARCHAR(4000),
    architecture_name           VARCHAR(64),
    parent_channel_id           NUMERIC,
    parent_channel_name         VARCHAR(256),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemChannel
  ADD CONSTRAINT SystemChannel_pk PRIMARY KEY (mgm_id, system_id, channel_id);

-- Source: tables/common/SystemConfigChannel.sql
    
CREATE TABLE SystemConfigChannel
(
    mgm_id                  NUMERIC NOT NULL,
    system_id               NUMERIC NOT NULL,
    config_channel_id       NUMERIC NOT NULL,
    name                    VARCHAR(128),
    position                NUMERIC,
    synced_date             TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemConfigChannel
  ADD CONSTRAINT SystemConfigChannel_pk PRIMARY KEY (mgm_id, system_id, config_channel_id);

-- Source: tables/common/SystemEntitlement.sql
    
CREATE TABLE SystemEntitlement
(
    mgm_id                  NUMERIC NOT NULL,
    system_id               NUMERIC NOT NULL,
    system_group_id         NUMERIC NOT NULL,
    name                    VARCHAR(64),
    description             VARCHAR(1024),
    group_type              NUMERIC,
    group_type_name         VARCHAR(64),
    current_members         NUMERIC,
    organization            VARCHAR(128),
    synced_date             TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemEntitlement
  ADD CONSTRAINT SystemEntitlement_pk PRIMARY KEY (mgm_id, system_id, system_group_id);

-- Source: tables/common/SystemErrata.sql
    
CREATE TABLE SystemErrata
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    errata_id           NUMERIC NOT NULL,
    hostname            VARCHAR(128),
    advisory_name       VARCHAR(100),
    advisory_type       VARCHAR(32),
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemErrata
  ADD CONSTRAINT SystemErrata_pk PRIMARY KEY (mgm_id, system_id, errata_id);

-- Source: tables/common/SystemGroup.sql
    
CREATE TABLE SystemGroup
(
    mgm_id                    NUMERIC NOT NULL,
    system_id                 NUMERIC NOT NULL,
    system_group_id           NUMERIC NOT NULL,
    name                      VARCHAR(64),
    description               VARCHAR(1024),
    current_members           NUMERIC,
    organization              VARCHAR(128),
    synced_date               TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemGroup
  ADD CONSTRAINT SystemGroup_pk PRIMARY KEY (mgm_id, system_id, system_group_id);

-- Source: tables/common/SystemHistory.sql
    
CREATE TABLE SystemHistory
(
    mgm_id              NUMERIC NOT NULL,
    system_id           NUMERIC NOT NULL,
    history_id          NUMERIC NOT NULL,
    hostname            VARCHAR(128),
    event               VARCHAR(100),
    event_data          TEXT,
    event_time          TIMESTAMPTZ,
    synced_date         TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemHistory
  ADD CONSTRAINT SystemHistory_pk PRIMARY KEY (mgm_id, system_id, history_id);

-- Source: tables/common/SystemNetAddressV4.sql
    
CREATE TABLE SystemNetAddressV4
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    interface_id                NUMERIC NOT NULL,
    address                     VARCHAR(64) NOT NULL,
    netmask                     VARCHAR(64),
    broadcast                   VARCHAR(64),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemNetAddressV4
  ADD CONSTRAINT SystemNetAddressV4_pk PRIMARY KEY (mgm_id, system_id, interface_id, address);

-- Source: tables/common/SystemNetAddressV6.sql
    
CREATE TABLE SystemNetAddressV6
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    interface_id                NUMERIC NOT NULL,
    scope                       VARCHAR(64) NOT NULL,
    address                     VARCHAR(64) NOT NULL,
    netmask                     VARCHAR(64),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemNetAddressV6
  ADD CONSTRAINT SystemNetAddressV6_pk PRIMARY KEY (mgm_id, system_id, interface_id, scope, address);

-- Source: tables/common/SystemNetInterface.sql
    
CREATE TABLE SystemNetInterface
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    interface_id                NUMERIC NOT NULL,
    name                        VARCHAR(32),
    hardware_address            VARCHAR(96),
    module                      VARCHAR(128),
    primary_interface           BOOLEAN NOT NULL DEFAULT FALSE,
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemNetInterface
  ADD CONSTRAINT SystemNetInterface_pk PRIMARY KEY (mgm_id, system_id, interface_id);

-- Source: tables/common/SystemOutdated.sql
    
CREATE TABLE SystemOutdated
(
    mgm_id                      NUMERIC NOT NULL,
    system_id                   NUMERIC NOT NULL,
    packages_out_of_date        BIGINT,
    errata_out_of_date          BIGINT,
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemOutdated
  ADD CONSTRAINT SystemOutdated_pk PRIMARY KEY (mgm_id, system_id);

-- Source: tables/common/SystemVirtualData.sql
    
CREATE TABLE SystemVirtualData
(
    mgm_id                      NUMERIC NOT NULL,
    instance_id                 NUMERIC NOT NULL,
    host_system_id              NUMERIC,
    virtual_system_id           NUMERIC,
    name                        VARCHAR(128),
    instance_type_name          VARCHAR(128),
    vcpus                       NUMERIC,
    memory_size                 NUMERIC,
    uuid                        VARCHAR(128),
    confirmed                   NUMERIC(1,0),
    state_name                  VARCHAR(128),
    synced_date                 TIMESTAMPTZ DEFAULT (current_timestamp)
);

ALTER TABLE SystemVirtualData
  ADD CONSTRAINT SystemVirtualdata_pk PRIMARY KEY (mgm_id, instance_id);

-- Source: tables/common/VersionInfo.sql
    
CREATE TABLE VersionInfo
(
    name      VARCHAR(256) NOT NULL,
    label     VARCHAR(64)  NOT NULL,
    version   VARCHAR(512) NOT NULL,
    release   VARCHAR(512) NOT NULL,
    created   TIMESTAMPTZ DEFAULT (current_timestamp) NOT NULL,
    modified  TIMESTAMPTZ DEFAULT (current_timestamp) NOT NULL
);

CREATE UNIQUE INDEX versioninfo_name_label_uq
    ON VersionInfo (name, label);

-- Source: tables/common/dual.sql
    

create table dual ( dummy char );

insert into dual values ( 'X' );

create or replace rule deny_insert_dual as on insert to dual do instead nothing;
create or replace rule deny_update_dual as on update to dual do instead nothing;
create or replace rule deny_delete_dual as on delete to dual do instead nothing;

-- Source: packages/rpm.pks
    
create schema rpm;


-- Source: packages/rpm.pkb
    
update pg_settings set setting = 'rpm,' || setting where name = 'search_path';

create or replace function isdigit(ch CHAR)
    RETURNS BOOLEAN as $$
    BEGIN
        if ascii(ch) between ascii('0') and ascii('9')
        then
            return TRUE;
        end if;
        return FALSE;
    END ;
$$ language 'plpgsql';

    
    create or replace FUNCTION isalpha(ch CHAR)
    RETURNS BOOLEAN as $$
    BEGIN
        if ascii(ch) between ascii('a') and ascii('z') or 
            ascii(ch) between ascii('A') and ascii('Z')
        then
            return TRUE;
        end if;
        return FALSE;
    END;
$$ language 'plpgsql';


    create or replace FUNCTION isalphanum(ch CHAR)
    RETURNS BOOLEAN as $$ 
    BEGIN
        if ascii(ch) between ascii('a') and ascii('z') or 
            ascii(ch) between ascii('A') and ascii('Z') or
            ascii(ch) between ascii('0') and ascii('9')
        then
            return TRUE;
        end if;
        return FALSE;
    END;
    $$ language 'plpgsql';


    create or replace FUNCTION rpmstrcmp (string1 IN VARCHAR, string2 IN VARCHAR)
    RETURNS INTEGER as $$
    declare
        str1 VARCHAR := string1;
        str2 VARCHAR := string2;
        digits VARCHAR(10) := '0123456789';
        lc_alpha VARCHAR(27) := 'abcdefghijklmnopqrstuvwxyz';
        uc_alpha VARCHAR(27) := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        alpha VARCHAR(54) := lc_alpha || uc_alpha;
        one VARCHAR;
        two VARCHAR;
        isnum BOOLEAN;
    BEGIN
        if str1 is NULL or str2 is NULL
        then
            RAISE EXCEPTION 'VALUE_ERROR.';
        end if;
      
        if str1 = str2
        then
            return 0;
        end if;
        one := str1;
        two := str2;

        <<segment_loop>>
        while one <> '' or two <> ''
        loop
            declare
                segm1 VARCHAR;
                segm2 VARCHAR;
                onechar CHAR(1);
                twochar CHAR(1);
            begin
                --raise notice 'Params: %, %',  one, two;
                -- Throw out all non-alphanum characters
                onechar := substr(one, 1, 1);
                twochar := substr(two, 1, 1);
                while one <> '' and not rpm.isalphanum(one) and onechar != '~' and onechar != '^'
                loop
                    one := substr(one, 2);
                end loop;
                while two <> '' and not rpm.isalphanum(two) and twochar != '~' and twochar != '^'
                loop
                    two := substr(two, 2);
                end loop;
                --raise notice 'new params: %, %', one, two;

                onechar := substr(one, 1, 1);
                twochar := substr(two, 1, 1);
                --raise notice 'new chars 1: %, %', onechar, twochar;
                /* handle the tilde separator, it sorts before everything else */
                if (onechar = '~' or twochar = '~')
                then
                    if (onechar != '~') then return 1; end if;
                    if (twochar != '~') then return -1; end if;
                    --raise notice 'passed tilde chars: %, %', onechar, twochar;
                    one := substr(one, 2);
                    two := substr(two, 2);
                    continue;
                end if;

                /*
                 * Handle caret separator. Concept is the same as tilde,
                 * except that if one of the strings ends (base version),
                 * the other is considered as higher version.
                 */
                onechar := substr(one, 1, 1);
                twochar := substr(two, 1, 1);
                --raise notice 'new chars 2: %, %', onechar, twochar;
                if (onechar = '^' or twochar = '^')
                then
                    if (one = '') then return -1; end if;
                    --raise notice 'passed caret chars 1: %, %', onechar, twochar;
                    if (two = '') then return 1; end if;
                    --raise notice 'passed caret chars 2: %, %', onechar, twochar;
                    if (onechar != '^') then return 1; end if;
                    --raise notice 'passed caret chars 3: %, %', onechar, twochar;
                    if (twochar != '^') then return -1; end if;
                    --raise notice 'passed caret chars 4: %, %', onechar, twochar;
                    one := substr(one, 2);
                    two := substr(two, 2);
                    continue;
                end if;

                if (not (one <> '' and two <> '')) then exit segment_loop; end if;

                str1 := one;
                str2 := two;
                if rpm.isdigit(str1) or rpm.isdigit(str2)
                then
                    str1 := ltrim(str1, digits);
                    str2 := ltrim(str2, digits);
                    isnum := true;
                else
                    str1 := ltrim(str1, alpha);
                    str2 := ltrim(str2, alpha);
                    isnum := false;
                end if;
                if str1 <> ''
                then segm1 := substr(one, 1, length(one) - length(str1));
                else segm1 := one;
                end if;

                if str2 <> ''
                then segm2 := substr(two, 1, length(two) - length(str2));
                else segm2 := two;
                end if;

                if isnum
                then
                    if segm1 = '' then return -1; end if;
                    if segm2 = '' then return 1; end if;

                    segm1 := ltrim(segm1, '0');
                    segm2 := ltrim(segm2, '0');

                    if segm1 = '' and segm2 <> ''
                    then
                        return -1;
                    end if;
                    if segm1 <> '' and segm2 = ''
                    then
                        return 1;
                    end if;
                    if length(segm1) < length(segm2) then return -1; end if;
                    if length(segm1) > length(segm2) then return 1; end if;
                end if;
                if segm1 < segm2 then return -1; end if;
                if segm1 > segm2 then return 1; end if;
               one := str1;
                two := str2;
            end;
        end loop segment_loop;
     
        if one = '' and two = '' then return 0; end if;
        if one = '' then return -1; end if;
        return 1;
    END ;
$$ language 'plpgsql';



   create or replace FUNCTION vercmp(
        e1 VARCHAR, v1 VARCHAR, r1 VARCHAR, 
        e2 VARCHAR, v2 VARCHAR, r2 VARCHAR)
    RETURNS INTEGER as $$
    declare
        rc INTEGER;
          ep1 INTEGER;
          ep2 INTEGER;
          BEGIN
            if e1 is null or e1 = '' then
              ep1 := 0;
            else
              ep1 := e1::integer;
            end if;
            if e2 is null or e2 = '' then
              ep2 := 0;
            else
              ep2 := e2::integer;
            end if;
            -- Epochs are non-null; compare them
            if ep1 < ep2 then return -1; end if;
            if ep1 > ep2 then return 1; end if;
            rc := rpm.rpmstrcmp(v1, v2);
            if rc != 0 then return rc; end if;
           return rpm.rpmstrcmp(r1, r2);
         END;
         $$ language 'plpgsql';

update pg_settings set setting = overlay( setting placing '' from 1 for (length('rpm')+1) ) where name = 'search_path';


-- Source: views/common/ErrataListReport.sql
    
CREATE OR REPLACE VIEW ErrataListReport AS
  SELECT Errata.mgm_id
            , Errata.errata_id
            , Errata.advisory_name
            , Errata.advisory_type
            , Errata.cve
            , Errata.synopsis
            , count(*) AS affected_systems
            , Errata.synced_date

    FROM SystemErrata
            INNER JOIN Errata ON SystemErrata.mgm_id = Errata.mgm_id AND SystemErrata.errata_id = Errata.errata_id

GROUP BY Errata.mgm_id
            , Errata.errata_id
            , Errata.advisory_name
            , Errata.advisory_type
            , Errata.cve
            , Errata.synopsis
            , Errata.synced_date
;

-- Source: views/common/HistoryReport.sql
    
CREATE OR REPLACE VIEW HistoryReport AS
      SELECT mgm_id
                , system_id
                , action_id AS event_id
                , hostname
                , event
                , completion_time AS event_time
                , status
                , event_data
                , synced_date
        FROM SystemAction

    UNION ALL

      SELECT mgm_id
                , system_id
                , history_id AS event_id
                , hostname
                , event
                , event_time
                , 'Done' AS status
                , event_data
                , synced_date
        FROM SystemHistory
;

-- Source: views/common/InventoryReport.sql
    
CREATE OR REPLACE VIEW InventoryReport AS
    -- CTEs to group all one to many relationship joining values with ; as separator
    WITH Entitlements AS (
        SELECT mgm_id, system_id, string_agg(system_group_id || ' - ' || name, ';') AS entitlements
          FROM systementitlement
      GROUP BY mgm_id, system_id
    ), Groups AS (
        SELECT mgm_id, system_id, string_agg(system_group_id || ' - ' || name, ';') AS system_groups
          FROM SystemGroup
      GROUP BY mgm_id, system_id
    ), ConfigChannels AS (
        SELECT mgm_id, system_id, string_agg(config_channel_id || ' - ' || name, ';') AS configuration_channels
          FROM SystemConfigChannel
      GROUP BY mgm_id, system_id
    ), Channels AS (
        SELECT mgm_id, system_id, string_agg(channel_id || ' - ' || name, ';') AS software_channels
          FROM SystemChannel
      GROUP BY mgm_id, system_id
    ), V6Addresses AS (
        SELECT mgm_id, system_id, interface_id, string_agg(address || ' (' || scope || ')', ';') AS ip6_addresses
          FROM SystemNetAddressV6
      GROUP BY mgm_id, system_id, interface_id
    )
    SELECT System.mgm_id
              , System.system_id
              , System.profile_name
              , System.hostname
              , System.minion_id
              , System.machine_id
              , System.registered_by
              , System.registration_time
              , System.last_checkin_time
              , System.kernel_version
              , System.organization
              , System.architecture
              , System.hardware
              , SystemNetInterface.name  AS primary_interface
              , SystemNetInterface.hardware_address AS hardware_address
              , SystemNetAddressV4.address AS ip_address
              , V6Addresses.ip6_addresses
              , ConfigChannels.configuration_channels
              , Entitlements.entitlements
              , Groups.system_groups
              , SystemVirtualdata.host_system_id AS virtual_host
              , SystemVirtualdata.virtual_system_id IS NULL AS is_virtualized
              , SystemVirtualdata.instance_type_name AS virt_type
              , Channels.software_channels
              , COALESCE(SystemOutdated.packages_out_of_date, (0)::bigint) AS packages_out_of_date
              , COALESCE(SystemOutdated.errata_out_of_date, (0)::bigint) AS errata_out_of_date
              , System.synced_date
      FROM System
              LEFT JOIN SystemVirtualdata ON ( System.mgm_id = SystemVirtualdata.mgm_id AND System.system_id = SystemVirtualdata.virtual_system_id )
              LEFT JOIN SystemOutdated ON ( System.mgm_id = SystemOutdated.mgm_id AND System.system_id = SystemOutdated.system_id )
              LEFT JOIN SystemNetInterface ON (System.mgm_id = SystemNetInterface.mgm_id AND System.system_id = SystemNetInterface.system_id AND SystemNetInterface.primary_interface)
              LEFT JOIN SystemNetAddressV4 ON (System.mgm_id = SystemNetAddressV4.mgm_id AND System.system_id = SystemNetAddressV4.system_id AND SystemNetInterface.interface_id = SystemNetAddressV4.interface_id)
              LEFT JOIN V6Addresses ON (System.mgm_id = V6Addresses.mgm_id AND System.system_id = V6Addresses.system_id AND SystemNetInterface.interface_id = V6Addresses.interface_id)
              LEFT JOIN Entitlements ON ( System.mgm_id = entitlements.mgm_id AND System.system_id = entitlements.system_id )
              LEFT JOIN Groups ON ( System.mgm_id = Groups.mgm_id AND System.system_id = Groups.system_id )
              LEFT JOIN ConfigChannels ON ( System.mgm_id = ConfigChannels.mgm_id AND System.system_id = ConfigChannels.system_id )
              LEFT JOIN Channels ON ( System.mgm_id = Channels.mgm_id AND System.system_id = Channels.system_id )
  ORDER BY System.mgm_id, System.system_id
;

-- Source: views/common/SystemInactivityReport.sql
    
CREATE OR REPLACE VIEW SystemInactivityReport AS
  SELECT mgm_id
            , system_id
            , profile_name AS system_name
            , organization
            , last_checkin_time
            , (current_timestamp - last_checkin_time) AS inactivity
            , synced_date
    FROM system
ORDER BY mgm_id, system_id, organization
;

-- Source: data/common/VersionInfo.sql
    
DELETE
  FROM VersionInfo
 WHERE label = 'schema' and name = 'uyuni-reportdb-schema';

INSERT
  INTO VersionInfo ( name, label, version, release )
VALUES ('uyuni-reportdb-schema', 'schema', '4.3.2', '1.6.uyuni1');

COMMIT;


-- Source: end.sql
    
create or replace function create_varnull_constriants() returns integer as $$
declare
    tabs record;
    total integer default 0;
begin

    for tabs in select 
        c.relname as "tab",
        a.attname as "col"
    from
        pg_catalog.pg_attribute a
        left outer join pg_catalog.pg_class c on a.attrelid = c.oid 
    where
        -- skip system columns
        a.attnum > 0
        -- skip dropped columns
        and not a.attisdropped
        -- filter only varchars
        and a.atttypid = 1043
        -- skip cols that already has this constraint
        and not exists (
            select 1 from pg_catalog.pg_constraint 
            where conname = 'vn_' || c.relname || '_' || a.attname
        )
        -- filter only tables owned by current user
        and a.attrelid in (
            select c.oid from pg_catalog.pg_class c
            where relkind = 'r' and pg_catalog.pg_table_is_visible(c.oid) and relowner = (
                select oid from pg_catalog.pg_authid where rolname = current_user
            )
        ) loop
        begin
            -- create constraint
            execute 'alter table ' || tabs.tab || ' add constraint vn_' ||
                tabs.tab || '_' || tabs.col || ' check (' || tabs.col || ' <> '''')';
            -- count them
        exception when others then
            total = total + 1;
            raise warning '% unable to create constraint for %.%', now(), tabs.tab, tabs.col;
        end;
    end loop;

    return total;
end;
$$ language plpgsql;

select create_varnull_constriants();


